﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Xunit;
using CoreLibrary;
using CoreLibrary.Core;
using CoreLibrary.Joueurs;
using CoreLibrary.Regles;

namespace UnitTesting
{
    public class PartieUT
    {
        [Fact]

        public void TestPremierConstructeurValide()
        {
            IRegles regle = new ReglesClassiques();

            Partie partie = new Partie(regle);
            Assert.Equal(regle, partie.Regles);
        }

        [Fact]
        public void Partie_Constructeur_De_Copie_Cree_Une_Nouvelle_Instance_Avec_Les_Memes_Donnees()
        {

            IRegles regles = new ReglesClassiques();
            Partie partieOriginale = new Partie(regles);


            FieldInfo? joueursField = typeof(Partie).GetField("joueurs", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldInfo? plateauxField = typeof(Partie).GetField("plateaux", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldInfo? courantField = typeof(Partie).GetField("courant", BindingFlags.NonPublic | BindingFlags.Instance);
            PropertyInfo? tourField = typeof(Partie).GetProperty("Tour", BindingFlags.Public | BindingFlags.Instance);
            PropertyInfo? termineField = typeof(Partie).GetProperty("Termine", BindingFlags.Public | BindingFlags.Instance);


            if (joueursField != null && plateauxField != null)
            {
                Dictionary<string, bool>? joueurs = joueursField.GetValue(partieOriginale) as Dictionary<string, bool>;
                if (joueurs != null)
                {
                    joueurs.Add("Joueur1", false);
                    joueurs.Add("Joueur2", true);
                }

                List<Plateau>? plateaux = plateauxField.GetValue(partieOriginale) as List<Plateau>;
                if (plateaux != null)
                {
                    plateaux.Add(new Plateau(4, 10));
                    plateaux.Add(new Plateau(4, 10));
                }

            

                if(tourField != null)
                    tourField.SetValue(partieOriginale, 5);

                if (termineField != null)
                    termineField.SetValue(partieOriginale, false);

            
                var partieCopiee = new Partie(partieOriginale);


                Dictionary<string, bool>? joueursCopie = joueursField.GetValue(partieCopiee) as Dictionary<string, bool>;
                List<Plateau>? plateauxCopie = plateauxField.GetValue(partieCopiee) as List<Plateau>;

                if (courantField != null && tourField != null && termineField != null)
                {
                    int? courantCopie = courantField.GetValue(partieCopiee) as int?;
                    int? tourCopie = tourField.GetValue(partieCopiee) as int?;
                    bool? termineCopie = termineField.GetValue(partieCopiee) as bool?;


                    if (joueursCopie != null && plateauxCopie != null && courantCopie != null && tourCopie != null && termineCopie != null && joueurs != null && plateaux != null)
                    {
                        Assert.Equal(joueurs.Count, joueursCopie.Count);
                        Assert.Equal(plateaux.Count, plateauxCopie.Count);
                        Assert.Equal(5, tourCopie.Value);
                        Assert.False(termineCopie.Value);
                        Assert.Equal(regles, partieCopiee.Regles);
                    }



                    if (joueurs != null && joueursCopie != null)
                    {
                        foreach (string joueur in joueurs.Keys)
                        {
                            Assert.True(joueursCopie.ContainsKey(joueur));
                            Assert.Equal(joueurs[joueur], joueursCopie[joueur]);
                        }
                    }

                    if (plateaux != null && plateauxCopie != null)
                    {
                        foreach (Plateau plateau in plateaux)
                        {
                            Assert.Contains(plateau, plateauxCopie);
                        }
                    }

                }
                    
            }
        }
    }
}
